#
#   Pagos Online
#
#   Author: Mauricio Mercado <maumercado (at) gmail.com>
#   with a great help of ingenieroariel
#
#
from django.conf.urls.defaults import patterns
from satchmo_store.shop.satchmo_settings import get_satchmo_setting

ssl = get_satchmo_setting('SSL', default_value=False)

urlpatterns = patterns('',
    (r'^$', 'payment.modules.pagosonline.views.pay_ship_info', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-step2'),
    (r'^confirm/$', 'payment.modules.pagosonline.views.confirm_info', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-step3'),
#    (r'^success/$', 'payment.views.checkout.success', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-success'),
#    (r'^failure/$', 'payment.views.checkout.failure', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-failure'),
    (r'^answer/$', 'payment.modules.pagosonline.views.answerpay', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-answer'),
    (r'^notify/$', 'payment.modules.pagosonline.views.notify_callback', {'SSL': ssl}, 'PAGOSONLINE_satchmo_checkout-notify_callback'),
    (r'^confirmorder/$', 'payment.views.confirm.confirm_free_order', {'SSL' : ssl, 'key' : 'PAGOSONLINE'}, 'PAGOSONLINE_satchmo_checkout_free-confirm')
)



#
#
#   Author: Mauricio Mercado <maumercado (at) gmail.com>
#   with a great help of ingenieroariel
#
#   Based on "Manual de Integracion Tradicional Pagos Online" ver. 3.1, 2009, PAGOSONLINE
#   For more information about integration look at http://ayuda.pagosonline.com/
#
# 
from livesettings import *
from django.utils.translation import ugettext_lazy as _

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_PAGOSONLINE',
    _('PAGOSONLINE Payment Module Settings'))

config_register_list(
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'pagosonline',
        ),
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'PAGOSONLINE',
        ),
    StringValue(PAYMENT_GROUP,
    	'LABEL',
    	description=_('English name for this group on the checkout screens'),
    	default = 'Pagos Online',
    	help_text = _('This will be passed to the translation utility'),
	),
    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^pagosonline/',
        ordering=20,
        ),
    BooleanValue(
        PAYMENT_GROUP,
        'LIVE',
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False,
        ordering=30,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_CURRENCY',
        description=_('Currency'),
        default='COP',
        choices=[
            ('EUR', _("Euros")),
            ('COP', _("Colombian Pesos")),
            ('GBP', _("British Pound")),
            ('MXN', _("Mexican Pesos")),
            ('USD', _("U.S. Dollar")),
            ('VEB', _("Strong Bolivar")),
            ],
        ordering=40,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_TITULAR',
        description=_('Merchant title'),
        help_text=_('Description of your shop which will be visible on payment confirmation screen'),
        ordering=60,
        ),

    # signature
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_SIGNATURE_CODE',
        description=_('Signature code'),
        help_text=_('Your secret code used to sign transaction data'),
        ordering=100,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_USERID_CODE',
        description=_('User ID'),
        help_text=_('Your userid provided by PAGOSONLINE'),
        ordering=200,
        ),

    # post url
    StringValue(
        PAYMENT_GROUP,
        'POST_URL',
        description=_('Post URL'),
        help_text=_('The PAGOSONLINE URL for transaction posting.'),
        default="https://gateway.pagosonline.net/apps/gateway/index.html",
        ordering=120,
        ),
    StringValue(
        PAYMENT_GROUP,
        'POST_TEST_URL',
        description=_('Post URL Test'),
        help_text=_('The PAGOSONLINE URL for transaction posting.'),
        default="https://gateway2.pagosonline.net/apps/gateway/index.html",
        ordering=120,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_URL_CALLBACK',
        description=_('Callback URL'),
        help_text=_('Callback URL for on-line notifications about payment progress'),
        default='PAGOSONLINE_satchmo_checkout-notify_callback',
        ordering=300,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_URL_OK',
        description=_('OK URL'),
        help_text=_('URL for customer to return after successful payment'),
        default='PAGOSONLINE_satchmo_checkout-answer',
        ordering=310,
        ),
#    StringValue(
#        PAYMENT_GROUP,
#        'MERCHANT_URL_KO',
#        description=_('Failure URL'),
#        help_text=_('URL for customer to return after payment failure'),
#        default='PAGOSONLINE_satchmo_checkout-failure',
#        ordering=320,
#        ),
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False,
	)
)



# THIS FILE IS GENERATED FROM setup.py in pagosonline
short_version = '0.5.1'
version = '0.5.1'
full_version = '0.5.1'
git_revision = '79a521ea0896938418094a90e9e9e2dfb505be90'
release = True

if not release:
    version = full_version



#
#
#   Author: Mauricio Mercado <maumercado (at) gmail.com>
#   with a great help of ingenieroariel
#
#   Based on "Manual de Integracion Tradicional Pagos Online" ver. 3.1, 2009, PAGOSONLINE
#   For more information about integration look at http://ayuda.pagosonline.com/
#
#

from datetime import datetime
from decimal import Decimal
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound, HttpResponseBadRequest
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.cache import never_cache
from livesettings import config_get_group, config_value
from payment.utils import get_processor_by_key
from payment.views import payship
from satchmo_store.contact.models import Contact
from satchmo_store.shop.models import Order, Cart
from satchmo_store.shop.satchmo_settings import get_satchmo_setting
from satchmo_utils.dynamic import lookup_url, lookup_template
from satchmo_utils.views import bad_or_missing
import logging
from django.views.decorators.csrf import csrf_exempt


try:
    from hashlib import md5
except ImportError:
    # python < 2.5
    from md5 import md5
log = logging.getLogger()

def pay_ship_info(request):
    return payship.base_pay_ship_info(
            request,
            config_get_group('PAYMENT_PAGOSONLINE'), payship.simple_pay_ship_process_form,
            'shop/checkout/pagosonline/pay_ship.html'
            )
pay_ship_info = never_cache(pay_ship_info)


def _resolve_local_url(payment_module, cfgval, ssl=False):
    try:
        return lookup_url(payment_module, cfgval.value, include_server=True, ssl=ssl)
    except urlresolvers.NoReverseMatch:
        return cfgval.value


def confirm_info(request):
    payment_module = config_get_group('PAYMENT_PAGOSONLINE')

    try:
        order = Order.objects.from_request(request)
        get_buyer_email = Contact.objects.filter(id=order.contact_id)
    except Order.DoesNotExist:
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return HttpResponseRedirect(url)

    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0:
        template = lookup_template(payment_module, 'shop/checkout/empty_cart.html')
        return render_to_response(template,
                                  context_instance=RequestContext(request))

    # Check if the order is still valid
    if not order.validate(request):
        context = RequestContext(request, {'message': _('Your order is no longer valid.')})
        return render_to_response('shop/404.html', context_instance=context)

    # Check if we are in test or real mode
    live = payment_module.LIVE.value
    if live:
        post_url = payment_module.POST_URL.value
        prueba = 0
    else:
        post_url = payment_module.POST_TEST_URL.value
        prueba = 1
    #
    # PAGOSONLINE system does not accept multiple payment attempts with the same refVenta, even
    # if the previous one has never been finished. The worse is that it does not display
    # any message which could be understood by an end user.
    #
    # If user goes to PAGOSONLINE page and clicks 'back' button (e.g. to correct contact data),
    # the next payment attempt will be rejected.
    #
    # To provide higher probability of refVenta uniqueness, we add YYYY:DD:MM:hh:mm:ss timestamp part
    # to the order id, separated by 'T' character in the following way:
    #
    #   refVenta: xxxxxxxTYYYYDDMMHHMMSS
    #
    now = datetime.now()
    xchg_order_id = "%dT%04d%02d%02d%02d%02d%02d" % (order.id, now.year, now.day, now.month, now.hour, now.minute, now.second)

    signature_code = payment_module.MERCHANT_SIGNATURE_CODE.value
    userId = payment_module.MERCHANT_USERID_CODE.value
    amount = "%.2f" % order.balance
    log.debug("Amount for confirm Info %s" % amount)
    coin = payment_module.MERCHANT_CURRENCY.value
    signature_data = '~'.join(
            map(str, (
                    signature_code,
                    userId,
                    xchg_order_id,
                    amount,
                    coin,
                    )))

    try:
        cartnumber = request.session['cart']
    except KeyError:
        log.debug("No cart number found %s", request.session)
    
    signature=md5(signature_data).hexdigest()
    template = lookup_template(payment_module, 'shop/checkout/pagosonline/confirm.html')

    url_callback = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_CALLBACK, ssl=get_satchmo_setting('SSL'))
    url_ans = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_OK)
#    url_ko = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_KO)
    
    try:
        request.user.email
        emailComprador = request.user.email
    except:
        emailComprador = get_buyer_email[0].email

    float_balance = float(order.balance)
    no_iva = float_balance/1.16
    iva = round(0.16*no_iva,2)
    
    log.debug("IVA = %f" % iva)

    ctx = {
        'live': live,
        'post_url': post_url,
        'coin': payment_module.MERCHANT_CURRENCY.value,
        'url_callback': url_callback,
        'url_ans': url_ans,
        'usuarioId': userId,
	    'order': order,
        'xchg_order_id': xchg_order_id,
        'amount': amount,
        'iva': iva,
        'signature': signature,
	    'prueba': prueba,
        'emailComprador': emailComprador,
        'default_view_tax': config_value('TAX', 'DEFAULT_VIEW_TAX'),
    }

    log.debug(ctx)

    return render_to_response(template, ctx, context_instance=RequestContext(request))
confirm_info = never_cache(confirm_info)

def answerpay(request):

    payment_module = config_get_group('PAYMENT_PAGOSONLINE')

    try:
        order = Order.objects.from_request(request)
    except Order.DoesNotExist:
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return HttpResponseRedirect(url)

    """
    This can be used to generate a receipt or some other confirmation
    """
    data = request.GET
    global estado
    global codigo

    estado = {
	'1': "Sin Abrir",
	'2': "Abierta",
	'4': "Pagada y Abonada",
	'5': "Cancelada",
	'6': "Rechazada",
	'7': "Validacion",
	'8': "Reversada",
	'9': "Reversada fraudulenta",
	'10': "Enviada a ente financiero",
	'11': "Capturando datos tarjeta de credito",
	'12': "Esperando confirmacion sistema PSE",
	'13': "Activa Debitos ACH",
	'14': "Confirmando pago Efecty",
	'15': "Impreso",
	'16': "Debito ACH Registrado",
	}

    codigo = {
	'1': "Transaccion aprobada",
	'2': "Pago cancelado por el usuario",
	'3': "Pago cancelado por el usuario durante validacion",
	'4': "Transaccion rechazada por la entidad",
	'5': "Transaccion declinada por la entidad",
	'6': "Fondos insuficientes",
	'7': "Tarjeta invalida",
	'8': "Acuda a su entidad",
	'9': "Tarjeta vencida",
	'10': "Tarjeta restringida",
	'11': "Discrecional POL",
	'12': "Fecha de expiracion o campo seg. Invalidos",
	'13': "Repita transaccion",
	'14': "Transaccion invalida",
	'15': "Transaccion en proceso de validacion",
	'16': "Combinacion usuario-contrasena invalidos",
	'17': "Monto excede maximo permitido por entidad",
	'18': "Documento de identificacion invalido",
	'19': "Transaccion abandonada capturando datos TC",
	'20': "Transaccion abandonada",
	'21': "Imposible reversar transaccion",
	'22': "Tarjeta no autorizada para realizar compras por internet.",
	'23': "Transaccion rechazada",
	'24': "Transaccion parcial aprobada",
	'25': "Rechazada por no confirmacion",
	'26': "Comprobante generado, esperando pago en banco",
	'9994': "Transaccion pendiente por confirmar",
	'9995': "Certificado digital no encontrado",
	'9996': "Entidad no responde",
	'9997': "Error de mensajeria con la entidad financiera",
	'9998': "Error en la entidad financiera",
	'9999': "Error no especificado",
	}

    tipo_pago = {
	'10': "VISA",
	'11': "MASTERCARD",
	'12': "AMEX",
	'22': "DINERS",
	'24': "Verified by VISA",
	'25': "PSE",
	'27': "VISA Debito",
	'30': "Efecty",
	'31': "Pago referenciado",
	}	
    
    try:    
        buyinfo = {
            'coin': payment_module.MERCHANT_CURRENCY.value,
            'ref_venta': data['ref_venta'],
            'order_idpagos': data['ref_pol'],
            'amount': data['valor'],
            'ivatra': data['iva'],
    	    'estadopol': estado[data['estado_pol']],
    	    'codigoresp': codigo[data['codigo_respuesta_pol']],
    	    'fechaprocesamiento': data['fecha_procesamiento'],
    	    'msg': data['mensaje'],
    	    'tipo_medio_pago': tipo_pago[data['medio_pago']],
    	    'emailComprador': data['emailComprador'],
    	    'order': order,
    	}   
    except KeyError:
        return HttpResponseRedirect('/') 
        
    try:
        cartnumber = request.session['cart']
        for cart in Cart.objects.filter(id=cartnumber):
            cart.empty()
    except KeyError:
        pass

    return render_to_response('shop/checkout/pagosonline/answer.html',
                              context_instance=RequestContext(request, buyinfo))

answerpay = never_cache(answerpay)

@csrf_exempt
def notify_callback(request):
    
    payment_module = config_get_group('PAYMENT_PAGOSONLINE')
    signature_code = payment_module.MERCHANT_SIGNATURE_CODE.value
    if payment_module.LIVE.value:
        log.debug("Live on %s", payment_module.KEY.value)
    else:
        log.debug("Test on %s", payment_module.KEY.value)
    data = request.POST

    try:
        sig_data = '~'.join(
            map(str,(signature_code,
                data['usuario_id'],
                data['ref_venta'],
                data['valor'],
                data['moneda'],
                data['estado_pol']
        )))
    
        sig_calc = md5(sig_data).hexdigest()
        if sig_calc != data['firma'].lower():
            log.error("Invalid signature. Received '%s', calculated '%s'. sig_data %s" % (data['firma'], sig_calc, sig_data))
            return HttpResponseBadRequest("Checksum error")

        xchg_order_id = data['ref_venta']
        try:
            order_id = xchg_order_id[:xchg_order_id.index('T')]
            log.debug("Order ID = %s" % order_id)
        except ValueError:
            log.error("Incompatible order ID: '%s'" % xchg_order_id)
            return HttpResponseNotFound("Order not found")
        try:
            order = Order.objects.get(id=order_id)
            log.debug("Order object status = %s" % order.id )
        except Order.DoesNotExist:
            log.error("Received data for nonexistent Order #%s" % order_id)
            return HttpResponseNotFound("Order not found")
        amount = data['valor']
        log.debug("Amount Received in POST %s" % amount)
        trans_id = codigo[data['codigo_respuesta_pol']]
    except KeyError:
        log.error("Received incomplete PAGOSONLINE transaction data")
        return HttpResponseBadRequest("Incomplete data")
    
    # success
    if int(data['codigo_respuesta_pol']) == 1:
        order.add_status(status='New') 
        processor = get_processor_by_key('PAYMENT_PAGOSONLINE')
        payment = processor.record_payment(
            order=order,
            amount=amount,
            transaction_id=trans_id)

    elif int(data['codigo_respuesta_pol']) == 15:
        order.add_status(status='In Process', notes=u"%s " % codigo[data['codigo_respuesta_pol']])
        processor = get_processor_by_key('PAYMENT_PAGOSONLINE')
        payment = processor.record_payment(
            order=order,
            amount=amount,
            transaction_id=trans_id)

    elif int(data['codigo_respuesta_pol']) == 26:
        order.add_status(status='In Process', notes=u"%s " % codigo[data['codigo_respuesta_pol']])
        processor = get_processor_by_key('PAYMENT_PAGOSONLINE')
        payment = processor.record_payment(
            order=order,
            amount=amount,
            transaction_id=trans_id)

    elif int(data['codigo_respuesta_pol']) == 24:
        order.add_status(status='Billed', notes=u"%s" % codigo[data['codigo_respuesta_pol']])
        processor = get_processor_by_key('PAYMENT_PAGOSONLINE')
        payment = processor.record_payment(
            order=order,
            amount=amount,
            transaction_id=trans_id)            

    elif int(data['codigo_respuesta_pol']) == 9994:        
        processor = get_processor_by_key('PAYMENT_PAGOSONLINE')
        payment = processor.record_payment(
            order=order,
            amount=amount,
            transaction_id=trans_id)

    else:
        order.add_status(status='Cancelled', notes=u"%s" % trans_id)

    return HttpResponse()



# Make the version available if there is a version.py file
try:
    from extract.version import version as __version__
    from extract.version import git_revision as __git_revision__
except ImportError:
    __version__ = "unknwon"
    __git_revision__ = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import subprocess

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

MAJOR = 0
MINOR = 5
MICRO = 1
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)

# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout = subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION

def write_version_py(filename='pagosonline/version.py'):
    cnt = """# THIS FILE IS GENERATED FROM setup.py in pagosonline
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
"""
    # Adding the git rev number needs to be done inside write_version_py(),
    # otherwise the import of pagosonline.version messes up the build under Python 3.
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('pagosonline/version.py'):
        # must be a source distribution, use existing version file
        GIT_REVISION = "Unknown"

    if not ISRELEASED:
        FULLVERSION += '.dev-' + GIT_REVISION[:7]

    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'full_version' : FULLVERSION,
                       'git_revision' : GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        a.close()

    return FULLVERSION


# Creates pagosonline/version.py and returns the full version
full_version = write_version_py()

setup(name          = 'pagosonline',
      version       = full_version,
      description   = 'Satchmo gateway for the Colombian pagosonline service',
      license       = 'BSD',
      keywords      = 'satchmo pagosonline',
      author        = 'Mauricio Mercado',
      author_email  = 'maumercado@gmail.com',
      url   = 'http://github.com/jeasas/pagosonline',
      long_description = read('README'),
      packages = ['pagosonline',],
      install_requires = [],
      classifiers   = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        ],
)
